@@ -84,7 +84,7 @@ static KAboutData createAboutData()
 OKULAR_EXPORT_PLUGIN( DjVuGenerator, createAboutData() )
 
 DjVuGenerator::DjVuGenerator( QObject *parent, const QVariantList &args )
-    : Okular::Generator( parent, args ), m_docInfo( 0 ), m_docSyn( 0 )
+    : Okular::Generator( parent, args ), m_docSyn( 0 )
 {
     setFeature( TextExtraction );
     setFeature( Threaded );
@@ -120,8 +120,6 @@ bool DjVuGenerator::doCloseDocument()
     m_djvu->closeFile();
     userMutex()->unlock();
 
-    delete m_docInfo;
-    m_docInfo = 0;
     delete m_docSyn;
     m_docSyn = 0;
 
@@ -136,48 +134,34 @@ QImage DjVuGenerator::image( Okular::PixmapRequest *request )
     return img;
 }
 
-const Okular::DocumentInfo * DjVuGenerator::generateDocumentInfo()
+Okular::DocumentInfo DjVuGenerator::generateDocumentInfo( const QSet<Okular::DocumentInfo::Key> &keys ) const
 {
-    if ( m_docInfo )
-        return m_docInfo;
+    Okular::DocumentInfo docInfo;
 
-    m_docInfo = new Okular::DocumentInfo();
-
-    m_docInfo->set( Okular::DocumentInfo::MimeType, "image/vnd.djvu" );
+    if ( keys.contains( Okular::DocumentInfo::MimeType ) )
+        docInfo.set( Okular::DocumentInfo::MimeType, "image/vnd.djvu" );
 
     if ( m_djvu )
     {
         // compile internal structure reading properties from KDjVu
-        QString title = m_djvu->metaData( "title" ).toString();
-        m_docInfo->set( Okular::DocumentInfo::Title, title.isEmpty() ? i18nc( "Unknown title", "Unknown" ) : title );
-        QString author = m_djvu->metaData( "author" ).toString();
-        m_docInfo->set( Okular::DocumentInfo::Author, author.isEmpty() ? i18nc( "Unknown author", "Unknown" ) : author );
-        QString editor = m_djvu->metaData( "editor" ).toString();
-        m_docInfo->set( "editor", editor.isEmpty() ? i18nc( "Unknown editor", "Unknown" ) : editor, i18n( "Editor" ) );
-        QString publisher = m_djvu->metaData( "publisher" ).toString();
-        m_docInfo->set( "publisher", publisher.isEmpty() ? i18nc( "Unknown publisher", "Unknown" ) : publisher, i18n( "Publisher" ) );
-        QString year = m_djvu->metaData( "year" ).toString();
-        m_docInfo->set( Okular::DocumentInfo::CreationDate, year.isEmpty() ? i18nc( "Unknown creation date", "Unknown" ) : year );
-        QString volume = m_djvu->metaData( "volume" ).toString();
-        m_docInfo->set( "volume", volume.isEmpty() ? i18nc( "Unknown volume information", "Unknown" ) : volume, i18n( "Volume" ) );
-        QString doctype = m_djvu->metaData( "documentType" ).toString();
-        m_docInfo->set( "documentType", doctype.isEmpty() ? i18nc( "Unknown type of document", "Unknown" ) : doctype, i18n( "Type of document" ) );
-        QVariant numcomponents = m_djvu->metaData( "componentFile" );
-        m_docInfo->set( "componentFile", numcomponents.type() != QVariant::Int ? i18nc( "Unknown number of component files", "Unknown" ) : numcomponents.toString(), i18n( "Component Files" ) );
-    }
-    else
-    {
-        m_docInfo->set( Okular::DocumentInfo::Title, i18nc( "Unknown title", "Unknown" ) );
-        m_docInfo->set( Okular::DocumentInfo::Author, i18nc( "Unknown author", "Unknown" ) );
-        m_docInfo->set( "editor", i18nc( "Unknown editor", "Unknown" ), i18n( "Editor" ) );
-        m_docInfo->set( "publisher", i18nc( "Unknown publisher", "Unknown" ), i18n( "Publisher" ) );
-        m_docInfo->set( Okular::DocumentInfo::CreationDate, i18nc( "Unknown creation date", "Unknown" ) );
-        m_docInfo->set( "volume", i18nc( "Unknown volume information", "Unknown" ), i18n( "Volume" ) );
-        m_docInfo->set( "documentType", i18nc( "Unknown type of document", "Unknown" ), i18n( "Type of document" ) );
-        m_docInfo->set( "componentFile", i18nc( "Unknown number of component files", "Unknown" ), i18n( "Component Files" ) );
+        if ( keys.contains( Okular::DocumentInfo::Author ) )
+            docInfo.set( Okular::DocumentInfo::Title, m_djvu->metaData( "title" ).toString() );
+        if ( keys.contains( Okular::DocumentInfo::Author ) )
+            docInfo.set( Okular::DocumentInfo::Author, m_djvu->metaData( "author" ).toString() );
+        if ( keys.contains( Okular::DocumentInfo::CreationDate ) )
+            docInfo.set( Okular::DocumentInfo::CreationDate, m_djvu->metaData( "year" ).toString() );
+        if ( keys.contains( Okular::DocumentInfo::CustomKeys ) )
+        {
+            docInfo.set( "editor", m_djvu->metaData( "editor" ).toString(), i18n( "Editor" ) );
+            docInfo.set( "publisher", m_djvu->metaData( "publisher" ).toString(), i18n( "Publisher" ) );
+            docInfo.set( "volume", m_djvu->metaData( "volume" ).toString(), i18n( "Volume" ) );
+            docInfo.set( "documentType", m_djvu->metaData( "documentType" ).toString(), i18n( "Type of document" ) );
+            QVariant numcomponents = m_djvu->metaData( "componentFile" );
+            docInfo.set( "componentFile", numcomponents.type() != QVariant::Int ? i18nc( "Unknown number of component files", "Unknown" ) : numcomponents.toString(), i18n( "Component Files" ) );
+        }
     }
 
-    return m_docInfo;
+    return docInfo;
 }
 
 const Okular::DocumentSynopsis * DjVuGenerator::generateDocumentSynopsis()
