@@ -19,7 +19,6 @@
 #include <qlayout.h>
 #include <qmutex.h>
 #include <qregexp.h>
-#include <qstack.h>
 #include <qtextstream.h>
 #include <QtGui/QPrinter>
 #include <QtGui/QPainter>
@@ -421,9 +420,9 @@ static void PDFGeneratorPopplerDebugFunction(const QString &message, const QVari
 
 PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     : Generator( parent, args ), pdfdoc( 0 ),
-    docInfoDirty( true ), docSynopsisDirty( true ),
+    docSynopsisDirty( true ),
     docEmbeddedFilesDirty( true ), nextFontPage( 0 ),
-    annotProxy( 0 ), synctex_scanner( 0 )
+    annotProxy( 0 )
 {
     setFeature( Threaded );
     setFeature( TextExtraction );
@@ -462,18 +461,7 @@ Okular::Document::OpenResult PDFGenerator::loadDocumentWithPassword( const QStri
 #endif
     // create PDFDoc for the given file
     pdfdoc = Poppler::Document::load( filePath, 0, 0 );
-    Okular::Document::OpenResult success = init(pagesVector, password);
-    if (success == Okular::Document::OpenSuccess)
-    {
-        // no need to check for the existence of a synctex file, no parser will be
-        // created if none exists
-        initSynctexParser(filePath);
-        if ( !synctex_scanner && QFile::exists(filePath + QLatin1String( "sync" ) ) )
-        {
-            loadPdfSync(filePath, pagesVector);
-        }
-    }
-    return success;
+    return init(pagesVector, password);
 }
 
 Okular::Document::OpenResult PDFGenerator::loadDocumentFromDataWithPassword( const QByteArray & fileData, QVector<Okular::Page*> & pagesVector, const QString &password )
@@ -539,7 +527,6 @@ bool PDFGenerator::doCloseDocument()
     delete pdfdoc;
     pdfdoc = 0;
     userMutex()->unlock();
-    docInfoDirty = true;
     docSynopsisDirty = true;
     docSyn.clear();
     docEmbeddedFilesDirty = true;
@@ -547,11 +534,6 @@ bool PDFGenerator::doCloseDocument()
     docEmbeddedFiles.clear();
     nextFontPage = 0;
     rectsGenerated.clear();
-    if ( synctex_scanner )
-    {
-        synctex_scanner_free( synctex_scanner );
-        synctex_scanner = 0;
-    }
 
     return true;
 }
@@ -619,64 +601,47 @@ void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation,
     }
 }
 
-const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
+Okular::DocumentInfo PDFGenerator::generateDocumentInfo( const QSet<Okular::DocumentInfo::Key> &keys ) const
 {
-    if ( docInfoDirty )
+    Okular::DocumentInfo docInfo;
+    docInfo.set( Okular::DocumentInfo::MimeType, "application/pdf" );
+
+    userMutex()->lock();
+
+
+    if ( pdfdoc )
     {
-        userMutex()->lock();
-
-        docInfo.set( Okular::DocumentInfo::MimeType, "application/pdf" );
-
-        if ( pdfdoc )
-        {
-            // compile internal structure reading properties from PDFDoc
+        // compile internal structure reading properties from PDFDoc
+        if ( keys.contains( Okular::DocumentInfo::Title ) )
             docInfo.set( Okular::DocumentInfo::Title, pdfdoc->info("Title") );
+        if ( keys.contains( Okular::DocumentInfo::Subject ) )
             docInfo.set( Okular::DocumentInfo::Subject, pdfdoc->info("Subject") );
+        if ( keys.contains( Okular::DocumentInfo::Author ) )
             docInfo.set( Okular::DocumentInfo::Author, pdfdoc->info("Author") );
+        if ( keys.contains( Okular::DocumentInfo::Keywords ) )
             docInfo.set( Okular::DocumentInfo::Keywords, pdfdoc->info("Keywords") );
+        if ( keys.contains( Okular::DocumentInfo::Creator ) )
             docInfo.set( Okular::DocumentInfo::Creator, pdfdoc->info("Creator") );
+        if ( keys.contains( Okular::DocumentInfo::Producer ) )
             docInfo.set( Okular::DocumentInfo::Producer, pdfdoc->info("Producer") );
-            docInfo.set( Okular::DocumentInfo::CreationDate,
-                         KGlobal::locale()->formatDateTime( pdfdoc->date("CreationDate"), KLocale::LongDate, true ) );
-            docInfo.set( Okular::DocumentInfo::ModificationDate,
-                         KGlobal::locale()->formatDateTime( pdfdoc->date("ModDate"), KLocale::LongDate, true ) );
-
+        if ( keys.contains( Okular::DocumentInfo::CreationDate ) )
+            docInfo.set( Okular::DocumentInfo::CreationDate, KGlobal::locale()->formatDateTime( pdfdoc->date("CreationDate"), KLocale::LongDate, true ) );
+        if ( keys.contains( Okular::DocumentInfo::ModificationDate ) )
+            docInfo.set( Okular::DocumentInfo::ModificationDate, KGlobal::locale()->formatDateTime( pdfdoc->date("ModDate"), KLocale::LongDate, true ) );
+        if ( keys.contains( Okular::DocumentInfo::CustomKeys ) )
+        {
             int major, minor;
             pdfdoc->getPdfVersion(&major, &minor);
-            docInfo.set( "format", i18nc( "PDF v. <version>", "PDF v. %1.%2",
-                         major, minor ), i18n( "Format" ) );
-            docInfo.set( "encryption", pdfdoc->isEncrypted() ? i18n( "Encrypted" ) : i18n( "Unencrypted" ),
-                         i18n("Security") );
-            docInfo.set( "optimization", pdfdoc->isLinearized() ? i18n( "Yes" ) : i18n( "No" ),
-                         i18n("Optimized") );
-
-            docInfo.set( Okular::DocumentInfo::Pages, QString::number( pdfdoc->numPages() ) );
+            docInfo.set( "format", i18nc( "PDF v. <version>", "PDF v. %1.%2", major, minor ), i18n( "Format" ) );
+            docInfo.set( "encryption", pdfdoc->isEncrypted() ? i18n( "Encrypted" ) : i18n( "Unencrypted" ), i18n("Security") );
+            docInfo.set( "optimization", pdfdoc->isLinearized() ? i18n( "Yes" ) : i18n( "No" ), i18n("Optimized") );
         }
-        else
-        {
-            // TODO not sure one can reach here, check and if it is not possible, remove the code
-            docInfo.set( Okular::DocumentInfo::Title, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::Subject, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::Author, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::Keywords, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::Creator, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::Producer, i18n("Unknown") );
-            docInfo.set( Okular::DocumentInfo::CreationDate, i18n("Unknown Date") );
-            docInfo.set( Okular::DocumentInfo::ModificationDate, i18n("Unknown Date") );
 
-            docInfo.set( "format", "PDF", i18n( "Format" ) );
-            docInfo.set( "encryption", i18n( "Unknown Encryption" ), i18n( "Security" ) );
-            docInfo.set( "optimization", i18n( "Unknown Optimization" ), i18n( "Optimized" ) );
-
-            docInfo.set( Okular::DocumentInfo::Pages, i18n("Unknown") );
-        }
-        userMutex()->unlock();
-
-        // if pdfdoc is valid then we cached good info -> don't cache them again
-        if ( pdfdoc )
-            docInfoDirty = false;
+        docInfo.set( Okular::DocumentInfo::Pages, QString::number( pdfdoc->numPages() ) );
     }
-    return &docInfo;
+    userMutex()->unlock();
+
+    return docInfo;
 }
 
 const Okular::DocumentSynopsis * PDFGenerator::generateDocumentSynopsis()
@@ -1161,25 +1126,16 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
         Okular::DocumentViewport viewport;
         QString optionString = option.toString();
 
-        // if option starts with "src:" assume that we are handling a
-        // source reference
-        if ( optionString.startsWith( "src:", Qt::CaseInsensitive ) )
+        // asking for the page related to a 'named link destination'. the
+        // option is the link name. @see addSynopsisChildren.
+        userMutex()->lock();
+        Poppler::LinkDestination *ld = pdfdoc->linkDestination( optionString );
+        userMutex()->unlock();
+        if ( ld )
         {
-            fillViewportFromSourceReference( viewport, optionString );
-        }
-        else
-        {
-            // asking for the page related to a 'named link destination'. the
-            // option is the link name. @see addSynopsisChildren.
-            userMutex()->lock();
-            Poppler::LinkDestination *ld = pdfdoc->linkDestination( optionString );
-            userMutex()->unlock();
-            if ( ld )
-            {
-                fillViewportFromLinkDestination( viewport, *ld );
-            }
-            delete ld;
+            fillViewportFromLinkDestination( viewport, *ld );
         }
+        delete ld;
         if ( viewport.pageNumber >= 0 )
             return viewport.toString();
     }
@@ -1607,223 +1563,11 @@ void PDFGenerator::addFormFields( Poppler::Page * popplerPage, Okular::Page * pa
         page->setFormFields( okularFormFields );
 }
 
-struct pdfsyncpoint
-{
-    QString file;
-    qlonglong x;
-    qlonglong y;
-    int row;
-    int column;
-    int page;
-};
-
-void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*> & pagesVector )
-{
-    QFile f( filePath + QLatin1String( "sync" ) );
-    if ( !f.open( QIODevice::ReadOnly ) )
-        return;
-
-    QTextStream ts( &f );
-    // first row: core name of the pdf output
-    const QString coreName = ts.readLine();
-    // second row: version string, in the form 'Version %u'
-    QString versionstr = ts.readLine();
-    QRegExp versionre( "Version (\\d+)" );
-    versionre.setCaseSensitivity( Qt::CaseInsensitive );
-    if ( !versionre.exactMatch( versionstr ) )
-        return;
-
-    QHash<int, pdfsyncpoint> points;
-    QStack<QString> fileStack;
-    int currentpage = -1;
-    const QLatin1String texStr( ".tex" );
-    const QChar spaceChar = QChar::fromLatin1( ' ' );
-
-    fileStack.push( coreName + texStr );
-
-    QString line;
-    while ( !ts.atEnd() )
-    {
-        line = ts.readLine();
-        const QStringList tokens = line.split( spaceChar, QString::SkipEmptyParts );
-        const int tokenSize = tokens.count();
-        if ( tokenSize < 1 )
-            continue;
-        if ( tokens.first() == QLatin1String( "l" ) && tokenSize >= 3 )
-        {
-            int id = tokens.at( 1 ).toInt();
-            QHash<int, pdfsyncpoint>::const_iterator it = points.constFind( id );
-            if ( it == points.constEnd() )
-            {
-                pdfsyncpoint pt;
-                pt.x = 0;
-                pt.y = 0;
-                pt.row = tokens.at( 2 ).toInt();
-                pt.column = 0; // TODO
-                pt.page = -1;
-                pt.file = fileStack.top();
-                points[ id ] = pt;
-            }
-        }
-        else if ( tokens.first() == QLatin1String( "s" ) && tokenSize >= 2 )
-        {
-            currentpage = tokens.at( 1 ).toInt() - 1;
-        }
-        else if ( tokens.first() == QLatin1String( "p*" ) && tokenSize >= 4 )
-        {
-            // TODO
-            kDebug(PDFDebug) << "PdfSync: 'p*' line ignored";
-        }
-        else if ( tokens.first() == QLatin1String( "p" ) && tokenSize >= 4 )
-        {
-            int id = tokens.at( 1 ).toInt();
-            QHash<int, pdfsyncpoint>::iterator it = points.find( id );
-            if ( it != points.end() )
-            {
-                it->x = tokens.at( 2 ).toInt();
-                it->y = tokens.at( 3 ).toInt();
-                it->page = currentpage;
-            }
-        }
-        else if ( line.startsWith( QLatin1Char( '(' ) ) && tokenSize == 1 )
-        {
-            QString newfile = line;
-            // chop the leading '('
-            newfile.remove( 0, 1 );
-            if ( !newfile.endsWith( texStr ) )
-            {
-                newfile += texStr;
-            }
-            fileStack.push( newfile );
-        }
-        else if ( line == QLatin1String( ")" ) )
-        {
-            if ( !fileStack.isEmpty() )
-            {
-                fileStack.pop();
-            }
-            else
-                kDebug(PDFDebug) << "PdfSync: going one level down too much";
-        }
-        else
-            kDebug(PDFDebug).nospace() << "PdfSync: unknown line format: '" << line << "'";
-
-    }
-
-    QVector< QLinkedList< Okular::SourceRefObjectRect * > > refRects( pagesVector.size() );
-    foreach ( const pdfsyncpoint& pt, points )
-    {
-        // drop pdfsync points not completely valid
-        if ( pt.page < 0 || pt.page >= pagesVector.size() )
-            continue;
-
-        // magic numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
-        Okular::NormalizedPoint p(
-            ( pt.x * dpi().width() ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
-            ( pt.y * dpi().height() ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
-            );
-        QString file = pt.file;
-        Okular::SourceReference * sourceRef = new Okular::SourceReference( file, pt.row, pt.column );
-        refRects[ pt.page ].append( new Okular::SourceRefObjectRect( p, sourceRef ) );
-    }
-    for ( int i = 0; i < refRects.size(); ++i )
-        if ( !refRects.at(i).isEmpty() )
-            pagesVector[i]->setSourceReferences( refRects.at(i) );
-}
-
-void PDFGenerator::initSynctexParser( const QString& filePath )
-{
-    synctex_scanner = synctex_scanner_new_with_output_file( QFile::encodeName( filePath ), 0, 1);
-}
-
-const Okular::SourceReference * PDFGenerator::dynamicSourceReference( int pageNr, double absX, double absY )
-{
-    if  ( !synctex_scanner )
-        return 0;
-
-    if (synctex_edit_query(synctex_scanner, pageNr + 1, absX * 72. / dpi().width(), absY * 72. / dpi().height()) > 0)
-    {
-        synctex_node_t node;
-        // TODO what should we do if there is really more than one node?
-        while (( node = synctex_next_result( synctex_scanner ) ))
-        {
-            int line = synctex_node_line(node);
-            int col = synctex_node_column(node);
-            // column extraction does not seem to be implemented in synctex so far. set the SourceReference default value.
-            if ( col == -1 )
-            {
-                col = 0;
-            }
-            const char *name = synctex_scanner_get_name( synctex_scanner, synctex_node_tag( node ) );
-
-            Okular::SourceReference * sourceRef = new Okular::SourceReference( QFile::decodeName( name ), line, col );
-            return sourceRef;
-        }
-    }
-    return 0;
-}
-
 PDFGenerator::PrintError PDFGenerator::printError() const
 {
     return lastPrintError;
 }
 
-void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & viewport, const QString & reference ) const
-{
-    if ( !synctex_scanner )
-        return;
-
-    // The reference is of form "src:1111Filename", where "1111"
-    // points to line number 1111 in the file "Filename".
-    // Extract the file name and the numeral part from the reference string.
-    // This will fail if Filename starts with a digit.
-    QString name, lineString;
-    // Remove "src:". Presence of substring has been checked before this
-    // function is called.
-    name = reference.mid( 4 );
-    // split
-    int nameLength = name.length();
-    int i = 0;
-    for( i = 0; i < nameLength; ++i )
-    {
-        if ( !name[i].isDigit() ) break;
-    }
-    lineString = name.left( i );
-    name = name.mid( i );
-    // Remove spaces.
-    name = name.trimmed();
-    lineString = lineString.trimmed();
-    // Convert line to integer.
-    bool ok;
-    int line = lineString.toInt( &ok );
-    if (!ok) line = -1;
-
-    // Use column == -1 for now.
-    if( synctex_display_query( synctex_scanner, QFile::encodeName(name), line, -1 ) > 0 )
-    {
-        synctex_node_t node;
-        // For now use the first hit. Could possibly be made smarter
-        // in case there are multiple hits.
-        while( ( node = synctex_next_result( synctex_scanner ) ) )
-        {
-            // TeX pages start at 1.
-            viewport.pageNumber = synctex_node_page( node ) - 1;
-
-            if ( !viewport.isValid() ) return;
-
-            // TeX small points ...
-            double px = (synctex_node_visible_h( node ) * dpi().width()) / 72.27;
-            double py = (synctex_node_visible_v( node ) * dpi().height()) / 72.27;
-            viewport.rePos.normalizedX = px / document()->page(viewport.pageNumber)->width();
-            viewport.rePos.normalizedY = ( py + 0.5 ) / document()->page(viewport.pageNumber)->height();
-            viewport.rePos.enabled = true;
-            viewport.rePos.pos = Okular::DocumentViewport::Center;
-
-            return;
-        }
-    }
-}
-
 QWidget* PDFGenerator::printConfigurationWidget() const
 {
     if ( !pdfOptionsPage )
